/*  This program was created by Tristan Losier, and is released into the public domain.
    
    If you are using my original design, this program should be quite accurate. 'count' increments
    every second, and from my testing does not vary even slightly over the course of several hours.
    Meaning? If you specify the buzzer to go off after x hours, it should go off after exactly x
    hours, down to the second. Of course, I offer no warranty as to this programs accuracy.
    
    The value in OCR0A will determine the frequency of the tone the speaker emits. A value of 30
    should provide a tone quite close to 17 KHz, if you want something else you can play around with
    it. Don't use values over 255. A general formula to specify an exact frequency goes like this:
    
    OCR0A = (F_CPU / 8) / tone_freq
    
    F_CPU is 4194304 Hz, so for a tone of 17000 Hz we have: (4194304 / 8) / 17000 = 30.84. We round
    off the decimal because OCR0A can't handle them. The bigger the value you put into OCR0A, the
    lower the tone's pitch.
*/

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

uint32_t count = 0;     // When Timer1 is activated, this value should increment every second

  // Turn off the AVR to conserve power
void stop(void) {
    TCCR0A = TCCR0B = 0;            // Turn off both timers
    TCCR1 = TCCR1 = 0;
    PORTB &= ~_BV(PB0);             // Turn off PB0 (the speaker)
    MCUCR = _BV(SM1) | _BV(SE);     // Power down mode,  SE=1 (bit 5) -- enables Sleep Modes
    sleep_cpu();                    // Put the CPU into Power Down Sleep Mode
}

void start_speaker(void) {
    OCR0A = 30;                         // The value Timer0 counts to (to provide a 17 KHz signal)
    TCCR0A = _BV(COM0A0) | _BV(WGM01);  // Initalize Timer0
    TCCR0B = _BV(CS01);
                                        /*  COM0A0: toggle OC0A (PB2) when timer value (TCNT0) == OCR0A
                                            WGM01:  put timer in CTC (Clear on Timer Compare) mode
                                            CS01:   Start the timer prescaled tp F_CPU/8 */
}

void stop_speaker(void) {
    TCCR0A = TCCR0B = 0;            // Turn off Timer0
    PORTB &= ~_BV(PB0);             // Turn off PB0 (the speaker)
}

int main(void) {
    DDRB |= _BV(PB0);               // Set PB0 as an output (the speaker)
    PORTB &= ~_BV(PB0);             // Turn off PB0
    
    sei();                          // Enable interrupts
    TIMSK |= _BV(TOIE1);            // Enable the Timer1 overflow interrupt
    TCCR1 = _BV(CS13) | _BV(CS12) | _BV(CS11) | _BV(CS10);  // Initalize Timer1
                                    /*  CS13:10:  Start timer prescaled to F_CPU/16384 */

    while(1) {
        sleep_mode();               // Put AVR in sleep mode to conserve power
    }
}

  // Run when Timer1 overflows (every second)
ISR(TIM1_OVF_vect) {
    count++;
      // Start the beeping after 900 seconds (15 min), and stop it after 4500 seconds (1 hour and 15 min).
    if (count >= 900 && count <= 4500) {
          // This will cause the speakerto go off for 1 second, every 10 minutes.
        if (count % 600 == 0)
            start_speaker();
        else if (count % 600 == 1)
            stop_speaker();
    }
}
